/****************************************************************************
* (C) 1999-2005 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Germany
*  
*  http://www.beck-ipc.com
*
* ---------------------------------------------------------------------------
* Module        : PKTDRV.C
* Function      : Packet driver interface enables access to the ethernet
* ---------------------------------------------------------------------------

$Header: PKTDRV.C, 16, 07.07.2005 15:49:16, Jack Gatrost$

****************************************************************************/


/************************************************************************/
// Includes
/************************************************************************/
#include <DOS.H>

#include "config.h"
#include <PKTDRV.H>

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 Ethernet Packet Driver API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces PKTDRV\.H><efont><nl>
    <tab><linkto>#COMM<linkmid>Communication<linktail><tab>
    <tab><linkto>#COMM<linkmid>Configuration<linktail><tab> |
 @libcat1a  COMM | Communication | pkt02 | Set_Access_Type | Install an Access Handler |
 @libcat1a  COMM | Communication | pkt28 | Set_WildcardAccess_Type | Install an Wildcard Access handler |
 @libcat1a  COMM | Communication | pkt03 | Release_Type | Uninstall an Access Handler |
 @libcat1a  COMM | Communication | pkt04 | Send_Packet | Send an Ethernet Packet |

 @libcat2x  CONFIG | Config | pktAvail | PktDriver_Installed | Is the Packet Driver available |
 @libcat2x  CONFIG | Config | pkt01 | Get_Driver_Info | Get Driver Info |
 @libcat2x  CONFIG | Config | pkt14 | Set_Receive_Mode | Set the receive mode of the ethernet device |
 @libcat2x  CONFIG | Config | pkt15 | Get_Receive_Mode | Get the receive mode of the ethernet device |
 @libcat2x  CONFIG | Config | pkt16_27 | Set_Multicast | Set or Delete an Ethernet Multicast MAC Address |
 @libcat2x  CONFIG | Config | pkt06 | Get_Eth_Address | Get the @CHIPs Ethernet MAC Address |

*/

#ifdef _pkt02_
/*
 @doc DOC_pkt02_
 ---------------------------------------------------------------
 Install an Access Handler
 ---------------------------------------------------------------
 @clib  Set_Access_Type | Installs an access Handler for Ethernet Packets. |
        <cfont>unsigned int Set_Access_Type ( Driver_Info *drv_info,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; void far * Receiver,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; int *pkt_type,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; int *handle );<efont> |
 @parm  int  | drv_info | Pointer to struct returned from Get_Driver_Info
 @parm  int  | Receiver | Pointer to receiver handler function
 @parm  int  | pkt_type | Pointer to packet type (e.g. 0x0008 for IP)
 @parm  int  | handle   | Pointer where handle should be stored (required for release type)
 @rdesc 0 on Success<nl>
        else error code:<nl>
        <ul><code>
        0x00<tab>NO_ERROR  <nl>
        0x01<tab>BAD_HANDLE <tab><s2> // invalid handle number <nl>
        0x05<tab>BAD_TYPE  <tab><tab>  // Bad packet type specified<nl>
        0x09<tab>NO_SPACE   <tab><tab> // operation failed because of insufficient space <nl>
        0x0A<tab>TYPE_INUSE  <tab><s2> // the type had previously been accessed and not released<nl>
        0x0B<tab>BAD_COMMAND <s1><tab> // the command was out of range, or not implemented <nl>
        0x0C<tab>CANT_SEND  <tab><s3> // the packet couldn't be sent (usually a hardware error)<nl>
        <codee><ule>
 @comm  We support the following Ethernet V2.0 frame format: <ul>
        48 Bits (6 Bytes) Destination address<nl>
        48 Bits (6 Bytes) Source address<nl>
        16 Bits (2 Bytes) Type field e.g 0x0608 for ARP, 0x0008 for IP or user defined types<nl>
        46 to 1500 Bytes of user data.<ule>

        <s2> The maximum number of installed handlers is limited to five.<s2>
        In all RTOS variants except the TINY version, setting of ARP
        or IP handlers is not allowed here.
        <nl><nl>
        When an Ethernet packet of the type specified here at [DS:SI] is received by the
        the SC12's network device driver, this driver will perform callbacks to the
        receiver handler function specified here in ES:DI.<s2> This callback will be
        done twice:<ul>
        <li>First call:<ul>
        Input parameters to your handler: AX = 0, CX = received packet length<nl>
        Return Value from your handler: ES:DI - Pointer to buffer where driver can load
        the CX received bytes.<ule>
        <li>Second call:<ul>
        Input parameters to your handler: AX = 1, data ready in your buffer now<nl>
        Return Value from your handler: -- none --<ule>
        <ule>
        On the first call, your handler produces a buffer into which the driver can transfer
        the received packet.<s2> This byte transfer occurs  between the two calls to your
        handler function.<nl>
        Important: Because of our Little Endian processor you must exchange the bytes for the<nl>
        packet type e.g. use 0x0008 for the IP type instead of 0x0800.<nl>
 @Also  pkt01.htm | Get_Driver_Info  |
        pkt28.htm | Set_WildcardAccess_Type | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../pktdrv.htm#0xAE0x02<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/

/*************************************************************************/
//                 set access type
/*************************************************************************/
unsigned int Set_Access_Type(Driver_Info * drv_info, void far * Receiver, int far * pkt_type, int *handle)
{
   asm push ds
   _DL = drv_info->number;  /* Number */
   _AL = drv_info->_class;  /* Class */
   _BX = drv_info->type;    /* Type */
   _AH = 0x02;
   _CX = sizeof(int);       /* size of type, must be 2 */
   asm les di, Receiver     /* _ES = FP_SEG(Receiver); _DI = FP_OFF(Receiver); */
   asm lds si, pkt_type     /* _DS = FP_SEG(pkt_type); _SI = FP_OFF(pkt_type); */
   asm int PKTVECT
   asm pop ds

   if(_FLAGS&0x01)
   {
      return (unsigned)_DH;
   }
   else
   {
      *handle = _AX;
      return 0;
   }
}
#endif



/*
 @doc DOC_pkt03_
 ---------------------------------------------------------------
 Release an Access Handler
 ---------------------------------------------------------------
 @clib  Release_Type | Uninstalls an access Handler |
        <cfont>unsigned int Release_Type( unsigned int handle );<efont> |
 @parm  int  | handle | Handle of access handler which should be uninstalled (returned
        from Set_Access_Type).
 @rdesc 0 on Success<nl>
        else error code:<nl>
        <ul><code>
         0x00<tab>NO_ERROR  <nl>
         0x01<tab>BAD_HANDLE <tab><s2> // invalid handle number <nl>
         0x05<tab>BAD_TYPE  <tab><tab>  // Bad packet type specified<nl>
         0x09<tab>NO_SPACE   <tab><tab> // operation failed because of insufficient space <nl>
         0x0A<tab>TYPE_INUSE  <tab><s2> // the type had previously been accessed and not released<nl>
         0x0B<tab>BAD_COMMAND <s1><tab> // the command was out of range, or not implemented <nl>
         0x0C<tab>CANT_SEND  <tab><s3> // the packet couldn't be sent (usually a hardware error)<nl>
        <codee><ule>
 @Also  pkt02.htm | Set_Access_Type  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../pktdrv.htm#0xAE0x03<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/

#ifdef _pkt03_
unsigned int Release_Type(unsigned int handle)
{
  _BX = handle;
  _AH = 0x03;
  asm int PKTVECT
  if(_FLAGS&0x01)
  {
      return (unsigned)_DH;
  }
  return 0;
}
#endif

/*
 @doc DOC_pkt28_
 ---------------------------------------------------------------
 Install an Wildcard Access Handler
 ---------------------------------------------------------------
 @clib  Set_WildcardAccess_Type | Installs an wildcard packet access Handler for Ethernet Packets. |
        <cfont>unsigned int Set_WildcardAccess_Type ( Driver_Info *drv_info,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; void far * Receiver,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; int *pkt_type,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; int *handle );<efont> |
 @parm  int  | drv_info | Pointer to struct returned from Get_Driver_Info
 @parm  int  | Receiver | Pointer to receiver handler function
 @parm  int  | pkt_type | Pointer to pakcet type (e.g. 0xFFFF for accepting every ethernet packet)
 @parm  int  | handle   | Pointer where handle should be stored (required for release type)
 @rdesc 0 on Success<nl>
        else error code:<nl>
        <ul><code>
          0x00<tab>NO_ERROR  <nl>
          0x01<tab>BAD_HANDLE <tab><s2> // invalid handle number <nl>
          0x05<tab>BAD_TYPE  <tab><tab>  // Bad packet type specified<nl>
          0x09<tab>NO_SPACE   <tab><tab> // operation failed because of insufficient space <nl>
          0x0A<tab>TYPE_INUSE  <tab><s2> // the type had previously been accessed and not released<nl>
          0x0B<tab>BAD_COMMAND <s1><tab> // the command was out of range, or not implemented <nl>
          0x0C<tab>CANT_SEND  <tab><s3> // the packet couldn't be sent (usually a hardware error)<nl>
        <codee><ule>
 @comm  Only one wildcard could be installed. Installing a new type overwrites the previous.<nl>
        If a packettype of 0xFFFF is installed, any incoming packet will be accepted.<nl>
        Delete a wild card handle by installing a wildcard with the packet type 0x0000.<nl>
        "Normal" handlers installed with <linkto>#0xAE0x02<linkmid>call<linktail> are not overwritten by installing a wildcard<nl>
 @Also  pkt02.htm | Set_Access_Type  |
        pkt01.htm | Get_Driver_Info  |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../pktdrv.htm#0xAE0x28<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/
#ifdef _pkt28_
unsigned int Set_WildcardAccess_Type(Driver_Info * drv_info, void far * Receiver, int far * pkt_type, int *handle)
{
   asm push ds
   _DL = drv_info->number;  /* Number */
   _AL = drv_info->_class;  /* Class */
   _BX = drv_info->type;    /* Type */
   _CX = sizeof(int);       /* size of type, must be 2 */
   _AH = 0x28;
   asm les di, Receiver     /* _ES = FP_SEG(Receiver); _DI = FP_OFF(Receiver); */
   asm lds si, pkt_type     /* _DS = FP_SEG(pkt_type); _SI = FP_OFF(pkt_type); */
   asm int PKTVECT
   asm pop ds

   if(_FLAGS&0x01)
   {
      return (unsigned)_DH;
   }
   else
   {
      *handle = _AX;
      return 0;
   }
}
#endif



#ifdef _pkt04_
/*
 @doc DOC_pkt04_
 ---------------------------------------------------------------
 Send Packet
 ---------------------------------------------------------------
 @clib  Send_Packet | Sends an Ethernet Packet |
        <cfont>unsigned char Send_Packet ( const unsigned char *buffer,<nl>
        <tab><tab><tab><tab><tab><tab><tab>unsigned int len );<efont> |
 @parm  int | buffer | Pointer to packet buffer
 @parm  int | len    | Length of packet buffer
 @rdesc 0 on Success<nl>
        else error code:<nl>
        <ul><code>
         0x00<tab>NO_ERROR  <nl>
         0x01<tab>BAD_HANDLE <tab><s2> // invalid handle number <nl>
         0x05<tab>BAD_TYPE  <tab><tab>  // Bad packet type specified<nl>
         0x09<tab>NO_SPACE   <tab><tab> // operation failed because of insufficient space <nl>
         0x0A<tab>TYPE_INUSE  <tab><s2> // the type had previously been accessed and not released<nl>
         0x0B<tab>BAD_COMMAND <s1><tab> // the command was out of range, or not implemented <nl>
         0x0C<tab>CANT_SEND  <tab><s3> // the packet couldn't be sent (usually a hardware error)<nl>
        <codee><ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../pktdrv.htm#0xAE0x04<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/
unsigned char Send_Packet(const unsigned char far * buffer, unsigned int len)
{
    asm push ds
    _DS = FP_SEG(buffer);
    _SI = FP_OFF(buffer);
    _CX = len;
    _AH = 0x04;
    asm int PKTVECT
    asm pop ds

    if(_FLAGS&0x01)
    {
      return (unsigned)_DH;
    }
    return 0;
}
#endif


#ifdef _pktAvail_
/*
 @doc DOC_pktAvail_
 ---------------------------------------------------------------
 PktDriver_Installed
 ---------------------------------------------------------------
 @clib  PktDriver_Installed | Checks if the packet driver is available  |
        <cfont>unsigned char PktDriver_Installed ( void );<efont> |
 @parm  void | | -- none --
 @rdesc 1: installed<nl>
        0: not installed<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x16<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/
unsigned char PktDriver_Installed(void)
{
   _AH = 0x16;
   asm int 0xA0 // BIOS_VECT

   //check bit3 of dx
   if(_DX & 0x08)
   {
      return 1;
   }
   return 0;

}
#endif


#ifdef _pkt01_
/*
 @doc DOC_pkt01_
 ---------------------------------------------------------------
 Send Packet
 ---------------------------------------------------------------
 @clib  Get_Driver_Info | Get information about the packet driver |
        <cfont>unsigned char Get_Driver_Info( Driver_Info *drv_info );<efont> |
 @parm  int  | drv_info | Pointer to Driver_Info struct, where driver information should be stored
 @rdesc 0 on Success<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../pktdrv.htm#0xAE0x01<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/
unsigned char Get_Driver_Info(Driver_Info * drv_info )
{
     unsigned int  rbx;
     unsigned char ral;

     asm push ds
     _AH = 0x01;
     asm int PKTVECT
     asm pop ds

     if(_FLAGS&0x01) return _DH;
     rbx = _BX;                  // save BX
     ral = _AL;                  // save AL
     drv_info->version = rbx;
     drv_info->basic   = ral;
     drv_info->_class  = _CH;
     drv_info->number  = _CL;
     drv_info->type    = _DX;
     return 0;
}
#endif

#ifdef _pkt14_
/*
 @doc DOC_pkt14_
 ---------------------------------------------------------------
 Set Receive Mode
 ---------------------------------------------------------------
 @clib  Set_Receive_Mode | Set receive mode of the ethernet device |
        <cfont>unsigned char Set_Receive_Mode( unsigned int mode,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab>unsigned char *error );<efont> |
 @parm  int  | mode | receive mode
 @parm  int  | error | Pointer where error code should be stored
 @rdesc 0   on Success<nl>
        -1  invalid mode
 @comm  Settable modes:<ul>
        <code>
           RCV_BCAST //accept packets with own mac address or broadcast<nl>
           RCV_ALL<tab>//accept all incoming packets<nl>
        <codee><ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../pktdrv.htm#0xAE0x14<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/
unsigned int Set_Receive_Mode(unsigned int mode,unsigned char * error)
{
  *error=0;
  _CX = mode;
  _AH = 0x14;
  asm int PKTVECT

  if(_FLAGS&0x01)
  {
      *error=_DH;
      return 0;
  }
  return _AX;
}
#endif


#ifdef _pkt15_
/*
 @doc DOC_pkt15_
 ---------------------------------------------------------------
 Get Receive Mode
 ---------------------------------------------------------------
 @clib  Get_Receive_Mode | Get the current receive mode of the ethernet device |
        <cfont>unsigned char Get_Receive_Mode( unsigned char *error );<efont> |
 @parm  int | error | Pointer where error code should be stored
 @rdesc 0   on Success<nl>
        -1  invalid mode
 @comm  Possible Receive modes:<ul>
        <code>
          RCV_BCAST<tab><tab>//accept own address and broadcasts<nl>
          RCV_BCAST_MCAST //accept broadcasts and installed multicasts<nl>
          RCV_ALL<tab><tab>//all incoming packets are accepted<nl>
        <codee><ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../pktdrv.htm#0xAE0x15<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/
unsigned int Get_Receive_Mode(unsigned char * error)
{
  *error=0;
  _AH = 0x15;
  asm int PKTVECT

  if(_FLAGS&0x01)
  {
    *error=_DH;
    return 0;
  }
  return _AX;
}
#endif


#ifdef _pkt16_27_
/*
 @doc DOC_pkt16_27_
 ---------------------------------------------------------------
 Set Multicast
 ---------------------------------------------------------------
 @clib  Set_Multicast | Set ethernet multicast Address |
        <cfont>unsigned int Set_Multicast( unsigned char *mac_addr, int set );<efont> |
 @parm  int  | mac_addr | Pointer to multicast mac address buffer (6 bytes)
 @parm  int  | set | 1==set, 0==delete
 @rdesc 0 on Success<nl>
        else error code:<nl>
        <ul><code>
          0x00<tab>NO_ERROR  <nl>
          0x01<tab>BAD_HANDLE <tab><s2> // invalid handle number <nl>
          0x05<tab>BAD_TYPE  <tab><tab>  // Bad packet type specified<nl>
          0x09<tab>NO_SPACE   <tab><tab> // operation failed because of insufficient space <nl>
          0x0A<tab>TYPE_INUSE  <tab><s2> // the type had previously been accessed and not released<nl>
          0x0B<tab>BAD_COMMAND <s1><tab> // the command was out of range, or not implemented <nl>
          0x0C<tab>CANT_SEND  <tab><s3> // the packet couldn't be sent (usually a hardware error)<nl>
        <codee><ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../pktdrv.htm#0xAE0x16<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/
unsigned int Set_Multicast(unsigned char far * mac_addr, int set)
{

  _ES = FP_SEG(mac_addr);
  _DI = FP_OFF(mac_addr);
  _CX = 6;

  if(set)
  {
    _AH = 0x16;
  }
  else
  {
    _AH = 0x27;
  }
  asm int PKTVECT

  if(_FLAGS&0x01)
  {
      return  (unsigned int)_DH;
  }
  else
  {
      return 0;
  }
}
#endif


#ifdef _pkt06_
/*
 @doc DOC_pkt06_
 ---------------------------------------------------------------
 Get Ethernet Mac Address
 ---------------------------------------------------------------
 @clib  Get_Eth_Address | Get the @CHIP Ethernet MAC Address |
        <cfont>unsigned char Get_Eth_Address( unsigned char *buffer,<nl>
        <tab><tab><tab><tab><tab><tab><s3><tab>unsigned int len );<efont> |
 @parm  int  | buffer | Pointer to 6 byte buffer where the MAC Address should be stored
 @parm  int  | len | Length of the buffer (must be 6)
 @rdesc 0 on Success<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../pktdrv.htm#0xAE0x06<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | n/a   | V0.90
*/
unsigned char Get_Eth_Address(unsigned char far *buffer, unsigned int len)
{
   len = len; /* avoid compiler warning */

   _ES = FP_SEG(buffer);
   _DI = FP_OFF(buffer);
   _CX = 6;
   _AH = 0x06;
   asm int PKTVECT
   return 0;
}
#endif

/************************************************************************/
//end pktdrv.c
/************************************************************************/
